//MACROECONOMICS 3 2019-2020
//DYNARE CODE FOR SOLVING AND SIMULATING THE SIMPLE RBC MODEL
//CALIBRATION AND LOG-LINEARIZATION IS DONE EXACTLY AS IN KING AND REBELO 
//(1999): n=0, all variables in log but the net return of capital
//-------------------------------------------------------------------------
//1.Declaring variables and parameters
//-------------------------------------------------------------------------
//1.1. Endogenous variables and exogenous states
var y              //output
    c              //consumption
    x              //investment
    h              //hours worked
    lab_prod       //labor productivity
    w              //wage rate
    r              //net return from capital
    a              //total factor productivity (exogenous state)
    rk             //capital rental rate
    k              //capital stock
    l;             //leisure
//-------------------------------------------------------------------------
//1.2. Stochastic innovations (exogenous variables)
varexo eps;
//-------------------------------------------------------------------------
//1.3. Endogenous state (predetermined)
predetermined_variables k;
//-------------------------------------------------------------------------
parameters alph  A n g delta rho sig_e bet xi sig eta;
//-------------------------------------------------------------------------
//2. Assign numerical values to the parameters (calibration)
//2.1 load previously saved files 
load params;
set_param_value('alph',alpha);
set_param_value('A',A);
set_param_value('n',n);
set_param_value('g',g);
set_param_value('delta',delta);
set_param_value('rho',rho);
set_param_value('sig_e',sigma_e);
set_param_value('bet',beta);
set_param_value('xi',xi);
set_param_value('sig', sigma);
set_param_value('eta', eta);
//-------------------------------------------------------------------------
//2. The model: specify the equations characterizing the equilibrium 
//-------------------------------------------------------------------------
model;
//FIRMS BLOCK
(1-alph)*exp(y)/exp(k)=exp(rk);                                           //(1) optimal demand of capital 
alph*exp(y)/exp(h)=exp(w);                                                //(2) optimal demand of labor
exp(y)=exp(a)*exp(k)^(1-alph)*exp(h)^(alph);                              //(3) production function
//HOUSEHOLDS BLOCK
xi*(1-exp(h))^(-eta)=exp(w)*(exp(c))^(-sig);                              //(4) intra-temporal condition
(1+n)*(1+g)*exp(k(+1))=(1-delta)*exp(k)+exp(x);                           //(5) capital law of motion
exp(h)+exp(l)=1;                                                          //(6) time constraint 
r=exp(rk)-delta;                                                          //(7) net return from capital
exp(c)^(-sig)=bet/((1+g))*(1+r(+1))*exp(c(+1))^(-sig);                    //(8) inter-temporal condition
//MARKET CLEARING CONDITIONS AND DEFINITIONS
exp(y)=exp(c)+exp(x);                                                     //(9) goods market
exp(lab_prod)=exp(y)/exp(h);                                              //(10) labor productivity
//EXOGENOUS STATES
a=(1-rho)*log(A)+rho*a(-1)+eps;                                           //(11) technology shock
end;
//-------------------------------------------------------------------------
//3. Steady state: solve with the external file and check the solution 
//-------------------------------------------------------------------------
steady;
check;
//-------------------------------------------------------------------------
//4. Shocks (stochastic innovations)
//-------------------------------------------------------------------------
shocks;
var eps;
stderr (100*sig_e);
end;
//-------------------------------------------------------------------------
//5. Solve and simulate the model
//-------------------------------------------------------------------------
//stoch_simul;
stoch_simul(order=1,hp_filter=1600, nocorr);
//-------------------------------------------------------------------------
//Plots the impulse response functions (to have a 1% increase in the shock)
close all;
figure(1);
set(gca,'FontSize',16);
subplot(3,3,1);
plot(oo_.irfs.y_eps/(100*sig_e),'k','LineWidth',3);
title('Output');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,2);
plot(oo_.irfs.k_eps/(100*sig_e),'k','LineWidth',3);
title('Capital');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,3);
plot(oo_.irfs.c_eps/(100*sig_e),'k','LineWidth',3);
title('Consumption');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,4);
plot(oo_.irfs.h_eps/(100*sig_e),'k','LineWidth',3);
title('Labor');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,5);
plot(oo_.irfs.r_eps/(100*sig_e),'k','LineWidth',3);
title('Net return of capital');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,6);
plot(oo_.irfs.w_eps/(100*sig_e),'k','LineWidth',3);
title('Wage rate');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,7);
plot(oo_.irfs.x_eps/(100*sig_e),'k','LineWidth',3);
title('Investment');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,8);
plot(oo_.irfs.lab_prod_eps/(100*sig_e),'k','LineWidth',3);
title('Labor productivity');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);
subplot(3,3,9);
plot(oo_.irfs.a_eps/(100*sig_e),'k','LineWidth',3);
title('TFP');
xlabel('Quarter after the shock');
set(gca,'FontSize',16);








